@@ -1,4 +1,4 @@
-/* $Id: base64.h,v 1.8 2004/03/20 05:39:40 relson Exp $ */
+/* $Id: base64.h,v 1.9 2004/10/08 22:31:54 m-a Exp $ */
 
 /*****************************************************************************
 
@@ -13,6 +13,6 @@ NAME:
 #include "word.h"
 
 uint	base64_decode(word_t *word);
-bool	base64_validate(word_t *word);
+bool	base64_validate(const word_t *word);
 
 #endif	/* HAVE_BASE64_H */
