@@ -1,11 +1,11 @@
-/* $Id: bogoconfig.c,v 1.174 2004/06/24 23:37:39 relson Exp $ */
+/* $Id: bogoconfig.c,v 1.192 2004/09/24 01:03:25 relson Exp $ */
 
 /*****************************************************************************
 
 NAME:
    bogoconfig.c -- process config file parameters
 
-   2003-02-12 - split out from config.c	
+   2003-02-12 - split out from config.c
 
 AUTHOR:
    David Relson <relson@osagesoftware.com>
@@ -36,7 +36,6 @@ Note: bogolexer also uses configfile.c.
 #include "common.h"
 
 #include <ctype.h>
-#include "getopt.h"
 #include <unistd.h>
 #include <stdlib.h>
 #include <string.h>
@@ -44,6 +43,7 @@ Note: bogolexer also uses configfile.c.
 
 #include "bogoconfig.h"
 #include "bogofilter.h"
+#include "bogohome.h"
 #include "bogoreader.h"
 #include "bool.h"
 #include "charset.h"
@@ -102,14 +102,14 @@ struct option long_options[] = {
     { "version",			N, 0, 'V' },
     { "classify-stdin",			N, 0, 'b' },
     { "bogofilter_dir",			R, 0, 'd' },
-    { "ham",				N, 0, 'e' },
+    { "nonspam-exits-zero",		N, 0, 'e' },
     { "help",				N, 0, 'h' },
     { "db_cachesize",			N, 0, 'k' },
     { "use-syslog",			N, 0, 'l' },
     { "register-ham",			N, 0, 'n' },
     { "passthrough",			N, 0, 'p' },
     { "register-spam",			N, 0, 's' },
-    { "update-as-classed",		N, 0, 'u' },
+    { "update-as-scored",		N, 0, 'u' },
     { "timestamp-date",			N, 0, 'y' },
     { "config-file",			R, 0, 'c' },
     { "no-config-file",			N, 0, 'C' },
@@ -183,6 +183,11 @@ void process_parameters(int argc, char **argv, bool warn_on_error)
     fpin = stdin;
     set_today();		/* compute current date for token age */
 
+#ifdef __EMX__
+    _response (&argc, &argv);	/* expand response files (@filename) */
+    _wildcard (&argc, &argv);	/* expand wildcards (*.*) */
+#endif
+
     process_arglist(argc, argv, PR_COMMAND, PASS_1_CLI);
     process_config_files(warn_on_error);
     process_arglist(argc, argv, PR_COMMAND, PASS_3_CLI);
@@ -268,14 +273,15 @@ static const char *help_text[] = {
     "help options:\n",
     "  -h,                       - print this help message.\n",
     "  -V, --version             - print version information and exit.\n",
-    "  -Q, --query               - query (display) bogofilter configuration.\n",
+    "  -Q, --query               - query (display) base bogofilter configuration.\n",
+    "  -QQ                       - display extended configuration info.\n",
     "classification options:\n",
     "  -p, --passthrough         - passthrough.\n",
     "  -e, --ham-true            - in -p mode, exit with code 0 when the mail is not spam.\n",
-    "  -u, --update-as-classified- classify message as spam or non-spam and register accordingly.\n",
+    "  -u, --update-as-scored    - score message as spam or non-spam and register accordingly.\n",
     "  -M, --clasify-mbox        - set mailbox mode.  Classify multiple messages in an mbox formatted file.\n",
-    "  -b, --clasify-stdin       - set streaming bulk mode. Process multiple messages whose filenames are read from STDIN.\n",
-    "  -B, --classify-files=list - set bulk mode. Process multiple messages named as files on the command line.\n",
+    "  -b, --clasify-stdin       - set streaming bulk mode. Process multiple messages (files or directories) read from STDIN.\n",
+    "  -B, --classify-files=list - set bulk mode. Process multiple messages (files or directories) named on the command line.\n",
     "  -R, --dataframe           - print an R data frame.\n",
     "registration options:\n",
     "  -s, --register-spam       - register message(s) as spam.\n",
@@ -293,10 +299,12 @@ static const char *help_text[] = {
     "  -I, --input-file=file     - read message from 'file' instead of stdin.\n",
     "  -O, --output-file=file    - save message to 'file' in passthrough mode.\n",
     "parameter options:\n",
-    "  -m,  --min_dev=v1, --robs=2, --robx=v3v1[,v2[,v3]]\n",
+    "  -mv1[,v2[,v3]]\n",
+    "      --min_dev=v1, --robs=v2, --robx=v3\n",
     "                            - set user defined min_dev, robs, and robx values.\n",
-    "  -o, --spam_cutoff=v1, --ha_cutoff=v2v1[,v2]\n",
-    "                           - set user defined spam and non-spam cutoff values.\n",
+    "  -ov1[,v2]\n",
+    "      --spam_cutoff=v1, --ham_cutoff=v2\n",
+    "                            - set user defined spam and non-spam cutoff values.\n",
     "info options:\n",
     "  -t, --terse               - set terse output mode.\n",
     "  -T, --fixed-terse-format  - set invariant terse output mode.\n",
@@ -312,9 +320,7 @@ static const char *help_text[] = {
     "  --bogofilter_dir                  directory for wordlists\n",
     "  --charset_default                 default character set\n",
     "  --db_cachesize                    Berkeley db cache in Mb\n",
-    "  --degen_enabled                   if no match, simplify\n",
-    "  --first_match                     if degen use first match\n",
-    "  --ham_cutoff                      nonspam if score below\n",
+    "  --ham_cutoff                      nonspam if score below this\n",
     "  --header_format                   spam header format\n",
     "  --log_header_format               header written to log\n",
     "  --log_update_format               logged on update\n",
@@ -322,7 +328,7 @@ static const char *help_text[] = {
     "  --ns_esf                          effective size factor for ham\n",
     "  --replace_nonascii_characters     substitute '?' if bit8 is 1\n",
     "  --robs                            Robinson's s parameter\n",
-    "  --robx                            Robinson's x\n",
+    "  --robx                            Robinson's x parameter\n",
     "  --sp_esf                          effective size factor for spam\n",
     "  --spam_cutoff                     spam if score above this\n",
     "  --spam_header_name                passthrough adds/replaces\n",
@@ -336,6 +342,7 @@ static const char *help_text[] = {
     "  --timestamp                       apply token timestamps\n",
     "  --unsure_subject_tag              like spam_subject_tag\n",
     "  --user_config_file                configuration file\n",
+    "  --wordlist                        specify wordlist parameters\n",
     "\n",
     "bogofilter is a tool for classifying email as spam or non-spam.\n",
     "\n",
@@ -373,7 +380,7 @@ static void print_version(void)
 		  progtype, version, ds_version_str(), PACKAGE);
 }
 
-#define	OPTIONS	":-:bBc:Cd:DefFghHI:k:lL:m:MnNo:O:pqQRrsStTuUvVx:X:y:"
+#define	OPTIONS	":-:bBc:Cd:DeFhHI:k:lL:m:MnNo:O:pqQRsStTuUvVx:X:y:"
 
 /** These functions process command line arguments.
  **
@@ -391,6 +398,9 @@ static void process_arglist(int argc, char **argv, priority_t precedence, int pa
 
     if (pass != PASS_1_CLI) {
 	optind = opterr = 1;
+#ifdef __EMX__
+	optind = 0;
+#endif
 	/* don't use #ifdef here: */
 #if HAVE_DECL_OPTRESET
 	optreset = 1;
@@ -450,6 +460,11 @@ void process_arg(int option, const char *name, const char *val, priority_t prece
 {
     switch (option)
     {
+    case '?':
+	if (pass == PASS_1_CLI)
+	    fprintf(stderr, "Unknown option '%s'.\n", name);
+	break;
+
     case 'b':
 	bulk_mode = B_STDIN;
 	fpin = NULL;	/* Ensure that input file isn't stdin */
@@ -497,7 +512,7 @@ void process_arg(int option, const char *name, const char *val, priority_t prece
 	break;
 
     case 'L':
-	logtag = val;
+	logtag = xstrdup(val);
 	/*@fallthrough@*/
 
     case 'l':
@@ -526,11 +541,12 @@ void process_arg(int option, const char *name, const char *val, priority_t prece
 	break;
 
     case 'Q':
-	query = true;
+	if (pass == PASS_1_CLI)
+	    query += 1;
 	break;
 
     case 'R':
-	Rtable = 1;
+	Rtable = true;
 	break;
 
     case 's':
@@ -546,7 +562,7 @@ void process_arg(int option, const char *name, const char *val, priority_t prece
 	break;
 	
     case 'U':
-	unsure_stats = true;
+	unsure_stats = (val == NULL) ? true : get_bool(name, val);
 	break;
 
     case 'v':
@@ -563,17 +579,21 @@ void process_arg(int option, const char *name, const char *val, priority_t prece
 	break;
 
     case 'y':		/* date as YYYYMMDD */
-	set_date( string_to_date(val) );
+    {
+	const char *str = val ? val : optarg;
+	YYYYMMDD date = string_to_date(str);
+	if (date != 0 && date < 19990000) {
+	    fprintf(stderr, "Date format for '-y' option is YYYYMMDD\n");
+	    exit(EX_ERROR);
+	}
+	set_date( date );
 	break;
-
+    }
+	
     case ':':
 	fprintf(stderr, "Option -%c requires an argument.\n", optopt);
 	exit(EX_ERROR);
 
-    case '?':
-	fprintf(stderr, "Unknown option '%s'.\n", name);
-	exit(EX_ERROR);
-
     case '-':
 	if (pass == PASS_3_CLI)
 	    process_config_option(val, true, precedence);
@@ -595,7 +615,7 @@ void process_arg(int option, const char *name, const char *val, priority_t prece
 	break;
 
     case 'H':
-	header_line_markup = false;
+	header_line_markup = (val == NULL) ? false : get_bool(name, val);
 	break;
 
     case 'k':
@@ -656,7 +676,7 @@ void process_arg(int option, const char *name, const char *val, priority_t prece
     case 'V':
 	print_version();
 	exit(EX_OK);
-	
+
     case O_BLOCK_ON_SUBNETS:		block_on_subnets = get_bool(name, val);			break;
     case O_CHARSET_DEFAULT:		charset_default = get_string(name, val);		break;
     case O_HEADER_FORMAT:		header_format = get_string(name, val);			break;
@@ -668,7 +688,7 @@ void process_arg(int option, const char *name, const char *val, priority_t prece
     case O_SPAM_HEADER_NAME:		spam_header_name = get_string(name, val);		break;
     case O_SPAM_SUBJECT_TAG:		spam_subject_tag = get_string(name, val);		break;
     case O_STATS_IN_HEADER:		stats_in_header = get_bool(name, val);			break;
-    case O_TERSE:			terse = get_bool(name, val);				break;	
+    case O_TERSE:			terse = get_bool(name, val);				break;
     case O_TERSE_FORMAT:		terse_format = get_string(name, val);			break;
     case O_THRESH_UPDATE:		get_double(name, val, &thresh_update);			break;
     case O_TIMESTAMP:			timestamp_tokens = get_bool(name, val);			break;
@@ -677,38 +697,57 @@ void process_arg(int option, const char *name, const char *val, priority_t prece
     }
 }
 
-#define YN(b) (b ? "yes" : "no")
-#define NB(b) (b ? b : "")
+#define	Q1	if (query >= 1)
+#define	Q2	if (query >= 2)
+
+#define YN(b) (b ? "Yes" : "No")
+#define NB(b) ((b != NULL && *b != '\0') ? b : "''")
 
 void query_config(void)
 {
-    fprintf(stdout, "# %s version %s\n", progname, version);
-    fprintf(stdout, "\n");
-    fprintf(stdout, "%-11s = %0.6f  # (%8.2e)\n", "robx", robx, robx);
-    fprintf(stdout, "%-11s = %0.6f  # (%8.2e)\n", "robs", robs, robs);
-    fprintf(stdout, "%-11s = %0.6f  # (%8.2e)\n", "min_dev", min_dev, min_dev);
-    fprintf(stdout, "%-11s = %0.6f  # (%8.2e)\n", "ham_cutoff", ham_cutoff, ham_cutoff);
-    fprintf(stdout, "%-11s = %0.6f  # (%8.2e)\n", "spam_cutoff", spam_cutoff, spam_cutoff);
-    fprintf(stdout, "%-11s = %0.6f  # (%8.2e)\n", "ns_esf", ns_esf, ns_esf);
-    fprintf(stdout, "%-11s = %0.6f  # (%8.2e)\n", "sp_esf", sp_esf, sp_esf);
-    fprintf(stdout, "\n");
-    fprintf(stdout, "%-17s = %s\n",    "block_on_subnets",    YN(block_on_subnets));
-    fprintf(stdout, "%-17s = %s\n",    "charset_default",     charset_default);
-    fprintf(stdout, "%-17s = %s\n",    "replace_nonascii_characters", YN(replace_nonascii_characters));
-    fprintf(stdout, "%-17s = %s\n",    "stats_in_header",     YN(stats_in_header));
-    fprintf(stdout, "%-17s = %0.6f\n", "thresh_update",       thresh_update);
-    fprintf(stdout, "%-17s = %s\n",    "timestamp",           YN(timestamp_tokens));
-    fprintf(stdout, "\n");
-    fprintf(stdout, "%-17s = %s\n", "terse",               YN(terse));
-    fprintf(stdout, "%-17s = %s\n", "spam_header_name",    spam_header_name);
-    fprintf(stdout, "%-17s = %s\n", "spam_subject_tag",    NB(spam_subject_tag));
-    fprintf(stdout, "%-17s = %s\n", "unsure_subject_tag",  NB(unsure_subject_tag));
-    fprintf(stdout, "%-17s = %s\n", "header_format",       header_format);
-    fprintf(stdout, "%-17s = %s\n", "terse_format",        terse_format);
-    fprintf(stdout, "%-17s = %s\n", "log_header_format",   log_header_format);
-    fprintf(stdout, "%-17s = %s\n", "log_update_format",   log_update_format);
-    display_tag_array("spamicity_tags   ", spamicity_tags);
-    display_tag_array("spamicity_formats", spamicity_formats);
+    Q1 fprintf(stdout, "# %s version %s\n", progname, version);
+    Q1 fprintf(stdout, "\n");
+    Q1 fprintf(stdout, "%-11s = %0.6f  # (%8.2e)\n", "robx", robx, robx);
+    Q1 fprintf(stdout, "%-11s = %0.6f  # (%8.2e)\n", "robs", robs, robs);
+    Q1 fprintf(stdout, "%-11s = %0.6f  # (%8.2e)\n", "min_dev", min_dev, min_dev);
+    Q1 fprintf(stdout, "%-11s = %0.6f  # (%8.2e)\n", "ham_cutoff", ham_cutoff, ham_cutoff);
+    Q1 fprintf(stdout, "%-11s = %0.6f  # (%8.2e)\n", "spam_cutoff", spam_cutoff, spam_cutoff);
+    Q1 fprintf(stdout, "%-11s = %0.6f  # (%8.2e)\n", "ns_esf", ns_esf, ns_esf);
+    Q1 fprintf(stdout, "%-11s = %0.6f  # (%8.2e)\n", "sp_esf", sp_esf, sp_esf);
+    Q1 fprintf(stdout, "\n");
+    Q1 fprintf(stdout, "%-17s = %s\n",    "block_on_subnets",    YN(block_on_subnets));
+    Q1 fprintf(stdout, "%-17s = %s\n",    "charset_default",     charset_default);
+    Q1 fprintf(stdout, "%-17s = %s\n",    "replace_nonascii_characters", YN(replace_nonascii_characters));
+    Q2 fprintf(stdout, "%-17s = %s\n",    "no-header-tags",      YN(header_line_markup));
+    Q1 fprintf(stdout, "%-17s = %s\n",    "stats_in_header",     YN(stats_in_header));
+    Q2 fprintf(stdout, "%-17s = %s\n",    "report-unsure",       YN(unsure_stats));
+    Q1 fprintf(stdout, "%-17s = %0.6f\n", "thresh_update",       thresh_update);
+    Q1 fprintf(stdout, "%-17s = %s\n",    "timestamp",           YN(timestamp_tokens));
+    Q2 fprintf(stdout, "%-17s = %d\n",    "timestamp-date",      today);
+    Q1 fprintf(stdout, "\n");
+    Q1 fprintf(stdout, "%-17s = %s\n", "spam_header_name",    spam_header_name);
+    Q1 fprintf(stdout, "%-17s = %s\n", "spam_subject_tag",    NB(spam_subject_tag));
+    Q1 fprintf(stdout, "%-17s = %s\n", "unsure_subject_tag",  NB(unsure_subject_tag));
+    Q2 fprintf(stdout, "%-18s = %s\n", "syslog-tag",          NB(logtag));
+    Q1 fprintf(stdout, "%-17s = %s\n", "header_format",       header_format);
+    Q1 fprintf(stdout, "%-17s = %s\n", "terse_format",        terse_format);
+    Q1 fprintf(stdout, "%-17s = %s\n", "log_header_format",   log_header_format);
+    Q1 fprintf(stdout, "%-17s = %s\n", "log_update_format",   log_update_format);
+    Q1 display_tag_array("spamicity_tags   ", spamicity_tags);
+    Q1 display_tag_array("spamicity_formats", spamicity_formats);
+
+    Q2 fprintf(stdout, "\n");
+
+    Q2 fprintf(stdout, "%-18s = %s\n", "no-config-file",   YN(suppress_config_file));
+    Q2 fprintf(stdout, "%-18s = %s\n", "config-file",      NB(config_file_name));
+    Q2 fprintf(stdout, "%-18s = %s\n", "user_config_file", NB(user_config_file));
+    Q2 fprintf(stdout, "\n");
+
+    Q2 fprintf(stdout, "%-18s = %s\n", "bogofilter_dir", bogohome);
+    Q2 display_wordlists("%-18s   ");
+    Q2 fprintf(stdout, "\n");
+
+    Q2 fprintf(stdout, "%-18s = %d\n", "db_cachesize",    db_cachesize);
 
     exit(EX_OK);
 }
