@@ -1,4 +1,4 @@
-/* $Id: bogoconfig.h,v 1.17 2004/04/20 14:36:59 m-a Exp $ */
+/* $Id: bogoconfig.h,v 1.20 2004/08/22 15:43:14 relson Exp $ */
 
 /*****************************************************************************
 
@@ -13,10 +13,8 @@ AUTHOR:
 #ifndef BOGOCONFIG_H
 #define BOGOCONFIG_H
 
-#include "system.h"
-
 #include "configfile.h"
-#include "getopt.h"
+#include "getopt.h"		/* for struct option */
 
 /* Global variables */
 
