@@ -1,4 +1,4 @@
-/* $Id: bogotune.c,v 1.143 2004/06/08 14:16:57 relson Exp $ */
+/* $Id: bogotune.c,v 1.167 2004/10/19 23:49:47 relson Exp $ */
 
 /*****************************************************************************
 
@@ -22,7 +22,7 @@ AUTHORS:
 **       2. discard words
 **    c. replace wordhashs with wordcnts
 **    d. de-allocate resident wordlist
-** 
+**
 ** 2. internal wordlist ("-D") flag
 **    a. read all messages
 **    b. distribute messages
@@ -77,6 +77,8 @@ AUTHORS:
 #define	LARGE_COUNT	40000
 
 #define	HAM_CUTOFF	0.10
+#define MIN_HAM_CUTOFF	0.10	/* minimum final ham_cutoff */
+#define MAX_HAM_CUTOFF	0.45	/* maximum final ham_cutoff */
 #define	MIN_CUTOFF	0.55	/* minimum cutoff  for set_thresh() */
 #define	WARN_MIN	0.50	/* warning minimum for set_thresh() */
 #define	WARN_MAX	0.99	/* warning maximum for set_thresh() */
@@ -85,7 +87,7 @@ AUTHORS:
 #define FP_CUTOFF	0.999
 #define	SCAN_CUTOFF	0.500	/* skip scans if cutoff is less or equal */
 
-#define	CHECK_PCT	0.01	/* for checking high scoring non-spam 
+#define	CHECK_PCT	0.01	/* for checking high scoring non-spam
 				** and low scoring spam */
 
 /* bogotune's default parameters */
@@ -103,6 +105,10 @@ AUTHORS:
 #define	MD_MAX_F	MD_MAX_C+MD_DLT_F
 #define	MD_DLT_F	0.014
 
+/* robx limits */
+#define RX_MIN		0.4
+#define RX_MAX		0.6
+
 enum e_verbosity {
     SUMMARY	   = 1,	/* summarize main loop iterations	*/
     TIME	   = 2, /* include timing info           	*/
@@ -111,6 +117,13 @@ enum e_verbosity {
     SCORE_DETAIL	/* verbosity level for printing scores	*/
 };
 
+typedef enum e_ds_loc {
+    DS_NONE	   = 0,	/* datastore locn not specified */
+    DS_ERR	   = 1,	/* error in datastore locn spec */
+    DS_DSK	   = 2,	/* datastore on disk */
+    DS_RAM	   = 4 	/* datastore in ram  */
+} ds_loc;
+
 #define	MOD(n,m)	((n) - (floor((n)/(m)))*(m))
 #define	ROUND(m,n)	floor((m)*(n)+.5)/(n)
 
@@ -120,13 +133,12 @@ enum e_verbosity {
 #define KEY(r)		((r)->fn + (((r)->co > 0.5) ? (r)->co : 0.99))
 #define	ESF_SEL(a,b)	(!esf_flag ? (a) : (b))
 
-#define	QPRINTF		if (!quiet) fprintf
-
 /* Global Variables */
 
 const char *progname = "bogotune";
 static char *ds_file;
 static char *ds_path;
+static ds_loc ds_flag = DS_NONE;
 
 static bool    bogolex = false;		/* true if convert input to msg-count format */
 static bool    esf_flag = true;		/* test ESF factors if true */
@@ -173,13 +185,13 @@ static void bt_exit(void);
 static void check_wordlist_path(void);
 static int  load_hook(word_t *key, dsv_t *data, void *userdata);
 static void load_wordlist(ds_foreach_t *hook, void *userdata);
-static void show_elapsed_time(int beg, int end, uint cnt, double val, 
+static void show_elapsed_time(int beg, int end, uint cnt, double val,
 			      const char *lbl1, const char *lbl2);
 static void write_msgcount_file(wordhash_t *wh);
 
 /* Function Definitions */
 
-static void bt_trap(void) {} 
+static void bt_trap(void) {}
 
 static int get_cnt(double fst, double lst, double amt)
 {
@@ -203,17 +215,21 @@ static data_t *seq_by_amt(double fst, double lst, double amt)
 
 static data_t *seq_canonical(double fst, double amt)
 {
+    uint c;
+    float v;
     uint i = 0;
     data_t *val = xcalloc(1, sizeof(data_t));
 
-    val->cnt = 5;
-    val->data = xcalloc(val->cnt, sizeof(double));
+    val->data = xcalloc(5, sizeof(double));
 
+    fst = max(fst, RX_MIN);
+    fst = min(fst, RX_MAX);
     val->data[i++] = fst;
-    val->data[i++] = fst - amt;
-    val->data[i++] = fst + amt;
-    val->data[i++] = fst - amt * 2;
-    val->data[i++] = fst + amt * 2;
+    for (c = 1; c <= 2; c += 1) {
+	v = fst - amt * c; if (v >= RX_MIN) val->data[i++] = v;
+	v = fst + amt * c; if (v <= RX_MAX) val->data[i++] = v;
+    }
+    val->cnt = i;
 
     return val;
 }
@@ -264,12 +280,12 @@ static void init_fine(double _rs, double _md, double _rx,
 
     s0 = max(_spex - 1.5,  0.0);
     s1 = min(_spex + 1.5, 20.0);
-    
+
     spexp = seq_by_amt(s0, ESF_SEL(s0,s1), 0.5);
-    
+
     s0 = max(_nsex - 1.5,  0.0);
     s1 = min(_nsex + 1.5, 20.0);
-    
+
     nsexp = seq_by_amt(s0, ESF_SEL(s0,s1), 0.5);
 }
 
@@ -284,20 +300,18 @@ static void print_parms(const char *label, const char *format, data_t *data)
     printf(")\n"); fflush(stdout);
 }
 
-static void print_all_parms(void)
+static void print_all_parms(uint r_count)
 {
     if (verbose >= PARMS) {
 	print_parms("rsval", "%6.4f", rsval);
 	print_parms("rxval", "%5.3f", rxval);
 	print_parms("mdval", "%5.3f", mdval);
-	print_parms("spexp", "%8.2e", spexp);
-	print_parms("nsexp", "%8.2e", nsexp);
+	print_parms("spexp", "%5.2f", spexp);
+	print_parms("nsexp", "%5.2f", nsexp);
     }
-    else if (verbose >= TIME)
+    if (verbose >= TIME)
 	printf("cnt: %u (rs: %u, rx: %u, md: %u spex: %u, nsex: %u)\n",
-	       rsval->cnt * rxval->cnt * mdval->cnt * spexp->cnt *
-	       nsexp->cnt, rsval->cnt, rxval->cnt, mdval->cnt,
-	       spexp->cnt, nsexp->cnt);
+	       r_count, rsval->cnt, rxval->cnt, mdval->cnt, spexp->cnt, nsexp->cnt);
 }
 
 static int compare_ascending(const void *const ir1, const void *const ir2)
@@ -377,14 +391,15 @@ static bool check_for_high_ns_scores(void)
 
     score_ns(ns_scores);	/* scores in descending order */
 
+    /* want at least 1 high scoring non-spam for FP determination */
     if (ns_scores[t-1] < SPAM_CUTOFF)
 	return false;
 
     if (!quiet) {
 	fprintf(stderr,
-	        "Warning: test messages include many high scoring nonspam.\n");
+		"Warning: test messages include many high scoring nonspam.\n");
 	fprintf(stderr,
-	        "         You may wish to reclassify them and rerun.\n");
+		"         You may wish to reclassify them and rerun.\n");
     }
 
     return true;
@@ -437,34 +452,35 @@ static bool check_for_low_sp_scores(void)
 {
     uint t = ceil(sp_cnt * check_percent);
 
-    score_sp(sp_scores);			/* get scores */
+    score_sp(sp_scores);	/* get scores */
 
+    /* low scoring spam may cause problems ... */
     if (sp_scores[t-1] > HAM_CUTOFF)
 	return false;
-    else {
-	if (!quiet) {
-	    fprintf(stderr,
-	        "Warning: test messages include many low scoring spam.\n");
-	    fprintf(stderr,
-	        "         You may wish to reclassify them and rerun.\n");
-	}
-	return true;
+
+    if (!quiet) {
+	fprintf(stderr,
+		"Warning: test messages include many low scoring spam.\n");
+	fprintf(stderr,
+		"         You may wish to reclassify them and rerun.\n");
     }
+
+    return true;
 }
 
 static void scoring_error(void)
 {
     int i;
-    
+
     if (quiet)
 	return;
 
     printf("    high ham scores:\n");
-    for (i = 0; i < 10 && ns_scores[i] > SPAM_CUTOFF; i += 1) 
+    for (i = 0; i < 10 && ns_scores[i] > SPAM_CUTOFF; i += 1)
 	printf("      %2d %8.6f\n", i+1, ns_scores[i]);
 
     printf("    low spam scores:\n");
-    for (i = 0; i < 10 && sp_scores[i] < HAM_CUTOFF; i += 1) 
+    for (i = 0; i < 10 && sp_scores[i] < HAM_CUTOFF; i += 1)
 	printf("      %2d %8.6f\n", i+1, sp_scores[i]);
 }
 
@@ -523,7 +539,7 @@ static double scale(uint cnt, uint lo, uint hi, double beg, double end)
 **	 spam_cutoff
 **
 ** As count increases from 500 to 4000 ...
-**	1) initial target percent drops from 1% to 0.25% 
+**	1) initial target percent drops from 1% to 0.25%
 **	2) initial minimum target increases from 5 to 10
 */
 
@@ -532,7 +548,7 @@ static void set_thresh(uint count, double *scores)
     uint   ftarget = 0;
     double cutoff, lgc;
 
-    score_ns(scores);				/* get scores */
+    score_ns(scores);		/* get scores */
 
 /*
 **	Use parabolic curve to fit data
@@ -686,7 +702,7 @@ static void distribute(int mode, tunelist_t *ns_or_sp)
     int good = mode == REG_GOOD;
     int bad  = 1 - good;
 
-    bool divvy = ds_file == NULL && user_robx < EPS;
+    bool divvy = ds_file == NULL && user_robx < EPS && !msg_count_file;
 
     wordhash_t *train = ns_and_sp->train;
     mlhead_t *msgs = ns_or_sp->msgs;
@@ -697,7 +713,7 @@ static void distribute(int mode, tunelist_t *ns_or_sp)
     int train_good = 0;
     int train_bad  = 0;
 
-    double ratio = scale(msgs->count,  
+    double ratio = scale(msgs->count,
 			 LIST_COUNT + TEST_COUNT,	/* small count */
 			 LIST_COUNT + LIST_COUNT,	/* large count */
 			 LIST_COUNT / TEST_COUNT,	/* small ratio */
@@ -731,7 +747,7 @@ static void distribute(int mode, tunelist_t *ns_or_sp)
     }
 
     if (verbose > 1)
-	printf("%s:  train_count = %d, score_count = %d\n",  
+	printf("%s:  train_count = %d, score_count = %d\n",
 	       good ? "ns" : "sp",
 	       train_count, score_count);
 
@@ -814,8 +830,8 @@ static void print_version(void)
 
 static void help(void)
 {
-    (void)fprintf(stderr, 
-                  "Usage:  %s [options] { -c config } { -d directory } -n non-spam-files -s spam-files\n", 
+    (void)fprintf(stderr,
+		  "Usage:  %s [options] { -c config } { -d directory } -n non-spam-files -s spam-files\n",
 		  progname);
     (void)fprintf(stderr,
 		  "\t  -h      - print this help message.\n"
@@ -829,14 +845,14 @@ static void help(void)
     (void)fprintf(stderr,
 		  "\t  -T num  - specify fp target value\n"
 		  "\t  -s file1 file2 ... - spam files\n"
-		  "\t  -n file1 file2 ... - non-spam files\n" 
+		  "\t  -n file1 file2 ... - non-spam files\n"
 		  "\t  -v      - increase level of verbose messages\n"
 		  "\t          - accept high scoring non-spam and low scoring spam\n"
 		  "\t  -q      - quiet (suppress warnings)\n"
 	);
     (void)fprintf(stderr,
 		  "\n"
-		  "%s (version %s) is part of the bogofilter package.\n", 
+		  "%s (version %s) is part of the bogofilter package.\n",
 		  progname, version);
 }
 
@@ -846,6 +862,11 @@ static int process_arglist(int argc, char **argv)
 
     bulk_mode = B_CMDLINE;
 
+#ifdef __EMX__
+    _response (&argc, &argv);	/* expand response files (@filename) */
+    _wildcard (&argc, &argv);	/* expand wildcards (*.*) */
+#endif
+
     while (--argc > 0) {
 	char *arg = *++argv;
 	count += 1;
@@ -861,9 +882,10 @@ static int process_arglist(int argc, char **argv)
 		case 'd':
 		    argc -= 1;
 		    ds_file = *++argv;
+		    ds_flag = (ds_flag == DS_NONE) ? DS_DSK : DS_ERR;
 		    break;
 		case 'D':
-		    ds_file = NULL;
+		    ds_flag = (ds_flag == DS_NONE) ? DS_RAM : DS_ERR;
 		    break;
 		case 'E':
 		    esf_flag ^= true;
@@ -913,10 +935,15 @@ static int process_arglist(int argc, char **argv)
 	    filelist_add( (run_type == REG_GOOD) ? ham_files : spam_files, arg);
     }
 
-    if (!bogolex && 
+    if (ds_flag == DS_ERR) {
+	fprintf(stderr, "Only one '-d dir' or '-D' option is allowed.\n");
+	exit(EX_ERROR);
+    }
+
+    if (!bogolex &&
 	(spam_files->count == 0 || ham_files->count == 0)) {
-	fprintf(stderr, 
-		"Bogotune needs both non-spam and spam messages sets for its testing.\n");
+	fprintf(stderr,
+		"Bogotune needs both non-spam and spam message sets for its parameter testing.\n");
 	exit(EX_ERROR);
     }
 
@@ -971,7 +998,7 @@ static int load_hook(word_t *key, dsv_t *data, void *userdata)
 
     if (strcmp((char *)key->text, ".MSG_COUNT") == 0)
 	set_msg_counts(data->goodcount, data->spamcount);
-	
+
     return 0;
 }
 
@@ -979,7 +1006,7 @@ static double get_robx(void)
 {
     double rx;
 
-    if (user_robx > 0.0) 
+    if (user_robx > 0.0)
 	rx = user_robx;
     else {
 	printf("Calculating initial x value...\n");
@@ -988,8 +1015,8 @@ static double get_robx(void)
 	verbose = -verbose;		/* enable bogofilter debug output */
     }
 
-    if (rx > 0.6) rx = 0.6;
-    if (rx < 0.4) rx = 0.4;
+    if (rx > RX_MAX) rx = RX_MAX;
+    if (rx < RX_MIN) rx = RX_MIN;
 
     printf("Initial x value is %8.6f\n", rx);
 
@@ -1011,16 +1038,16 @@ static void top_ten(result_t *sorted, uint n)
 
     printf("Top ten parameter sets from this scan:\n");
 
-    if (verbose)
-	printf("    ");
     printf("        rs     md    rx    spesf    nsesf    co     fp  fn   fppc   fnpc\n");
     for(f = false; !f; f = true) {
       for (i = j = 0; i < 10 && j < n;) {
  	result_t *r = &sorted[j++];
  	if (!f && r->fp != target) continue;
+	sp_esf = ESF_SEL(sp_esf, pow(0.75, r->sp_exp));
+	ns_esf = ESF_SEL(ns_esf, pow(0.75, r->ns_exp));
+
 	printf("%5u %6.4f %5.3f %5.3f %8.6f %8.6f %6.4f  %3u %3u  %6.4f %6.4f\n",
-	       r->idx, r->rs, r->md, r->rx,
-	       pow(0.75, r->sp_exp), pow(0.75, r->ns_exp), r->co, 
+	       r->idx, r->rs, r->md, r->rx, sp_esf, ns_esf, r->co,
 	       r->fp, r->fn, r->fp*100.0/ns_cnt, r->fn*100.0/sp_cnt);
 	++i;
       }
@@ -1036,7 +1063,7 @@ static void top_ten(result_t *sorted, uint n)
 
 /* get false negative */
 
-static int gfn(result_t *results, 
+static int gfn(result_t *results,
 	       uint rsi, uint mdi, uint rxi,
 	       uint spi, uint nsi)
 {
@@ -1045,7 +1072,7 @@ static int gfn(result_t *results,
     int fn = r->fn;
     if (r->fp != target) return INT_MAX;
     if (verbose > 100)
-	printf("   %2u, %2u, %2u, %2u, %2u, %2d\n", 
+	printf("   %2u, %2u, %2u, %2u, %2u, %2d\n",
 	       rsi, mdi, rxi, spi, nsi, fn);
     ncnt += 1;
     nsum += fn;
@@ -1156,10 +1183,10 @@ static void final_recommendations(bool skip)
     printf("Performing final scoring:\n");
 
     printf("Spam...  ");
-    score_sp(sp_scores);		/* get scores (in ascending order) */
+    score_sp(sp_scores);	/* get scores (in ascending order) */
 
     printf("Non-Spam...\n");
-    score_ns(ns_scores);		/* get scores (in descending order) */
+    score_ns(ns_scores);	/* get scores (in descending order) */
 
     for(m=0; m<10; ++m) printf("%8.6f %8.6f\n", sp_scores[m], ns_scores[m]);
 
@@ -1177,9 +1204,9 @@ static void final_recommendations(bool skip)
     printf("---cut---\n");
     printf("db_cachesize=%u\n", db_cachesize);
 
-    printf("robx=%8.6f\n", robx);
-    printf("min_dev=%5.3f\n", min_dev);
     printf("robs=%6.4f\n", robs);
+    printf("min_dev=%5.3f\n", min_dev);
+    printf("robx=%8.6f\n", robx);
     printf("sp_esf=%8.6f\n", sp_esf);
     printf("ns_esf=%8.6f\n", ns_esf);
 
@@ -1232,14 +1259,14 @@ static void final_recommendations(bool skip)
     if (!skip) {
 	uint s = ceil(sp_cnt * 0.002 - 1);
 	ham_cutoff = sp_scores[s];
-	if (ham_cutoff < 0.10) ham_cutoff = 0.10;
-	if (ham_cutoff > 0.45) ham_cutoff = 0.45;
+	if (ham_cutoff < MIN_HAM_CUTOFF) ham_cutoff = MIN_HAM_CUTOFF;
+	if (ham_cutoff > MAX_HAM_CUTOFF) ham_cutoff = MAX_HAM_CUTOFF;
     }
 
     printf("ham_cutoff=%5.3f\t\n", ham_cutoff);
     printf("---cut---\n");
     printf("\n");
-    
+
     if (skip)
 	final_warning();
 
@@ -1302,27 +1329,30 @@ static bool check_msg_counts(void)
     bool ok = true;
 
     if (msgs_good < LIST_COUNT || msgs_bad < LIST_COUNT) {
-	QPRINTF(stderr, 
-		"The wordlist contains %d non-spam and %d spam messages.\n"
-		"Bogotune must be run with at least %d of each.\n",
-		(int) msgs_good, (int) msgs_bad, LIST_COUNT);
+	if (!quiet)
+	    fprintf(stderr,
+		    "The wordlist contains %d non-spam and %d spam messages.\n"
+		    "Bogotune must be run with at least %d of each.\n",
+		    (int) msgs_good, (int) msgs_bad, LIST_COUNT);
 	ok = false;
     }
 
     if (msgs_bad < msgs_good / 5 ||
 	msgs_bad > msgs_good * 5) {
-	QPRINTF(stderr,
-		"The wordlist has a ratio of spam to non-spam of %0.1f to 1.0.\n"
-		"Bogotune requires the ratio be in the range of 0.2 to 5.\n",
-		msgs_bad / msgs_good);
+	if (!quiet)
+	    fprintf(stderr,
+		    "The wordlist has a ratio of spam to non-spam of %0.1f to 1.0.\n"
+		    "Bogotune requires the ratio be in the range of 0.2 to 5.\n",
+		    msgs_bad / msgs_good);
 	ok = false;
     }
 
     if (ns_cnt < TEST_COUNT || sp_cnt < TEST_COUNT) {
-	QPRINTF(stderr, 
-		"The messages sets contain %u non-spam and %u spam.  Bogotune "
-		"requires at least %d non-spam and %d spam messages to run.\n",
-		ns_cnt, sp_cnt, TEST_COUNT, TEST_COUNT);
+	if (!quiet)
+	    fprintf(stderr,
+		    "The messages sets contain %u non-spam and %u spam.  Bogotune "
+		    "requires at least %d non-spam and %d spam messages to run.\n",
+		    ns_cnt, sp_cnt, TEST_COUNT, TEST_COUNT);
 	exit(EX_ERROR);
     }
 
@@ -1374,8 +1404,8 @@ static rc_t bogotune(void)
     if (bogolex)
 	return status;
 
-    distribute(REG_SPAM, sp_msglists);
     distribute(REG_GOOD, ns_msglists);
+    distribute(REG_SPAM, sp_msglists);
 
     create_countlists(ns_msglists);
     create_countlists(sp_msglists);
@@ -1400,7 +1430,7 @@ static rc_t bogotune(void)
 
     fflush(stdout);
 
-    check_percent = CHECK_PCT;	/* for checking high scoring non-spam 
+    check_percent = CHECK_PCT;	/* for checking high scoring non-spam
 				** and low scoring spam */
 
     ns_scores = xcalloc(ns_cnt, sizeof(double));
@@ -1431,7 +1461,7 @@ static rc_t bogotune(void)
     ** shipped, and determine the count that will result from a spam cutoff
     ** of 0.95; if that is < 0.25%, try 0.9375 etc.
     */
-    
+
     min_dev = 0.02;
 
     /* set target and spam_cutoff */
@@ -1455,6 +1485,9 @@ static rc_t bogotune(void)
     }
 #endif
 
+    if (!esf_flag && (sp_esf < 1.0 || ns_esf < 1.0))
+	fprintf(stderr, "Warning:  Using ESF values (sp=%8.6f, ns=%8.6f) from config file.\n", sp_esf, ns_esf);
+
     /* No longer needed */
     wordhash_free(ns_and_sp->train);
     ns_and_sp->train = NULL;
@@ -1493,18 +1526,17 @@ static rc_t bogotune(void)
 	    break;
 	}
 
-	print_all_parms();
-
-	r_count = rsval->cnt * mdval->cnt * rxval->cnt * spexp->cnt
-	    * nsexp->cnt;
+	r_count = rsval->cnt * mdval->cnt * rxval->cnt * spexp->cnt * nsexp->cnt;
 	results = (result_t *) xcalloc(r_count, sizeof(result_t));
 
+	print_all_parms(r_count);
+
 	if (verbose >= SUMMARY) {
 	    if (verbose >= SUMMARY+1)
 		printf("%3s ", "cnt");
 	    if (verbose >= SUMMARY+2)
-		printf(" %s %s %s  ", "s", "m", "x");
-	    printf(" %4s %5s   %4s %8s %8s %7s %3s %3s\n", 
+		printf(" %s %s %s      ", "s", "m", "x");
+	    printf(" %4s %5s   %4s %8s %8s %7s %3s %3s\n",
 		   "rs", "md", "rx", "spesf", "nsesf", "cutoff", "fp", "fn");
 	}
 
@@ -1517,12 +1549,12 @@ static rc_t bogotune(void)
 	    for (rxi = 0; rxi < rxval->cnt; rxi++) {
 	      robx = rxval->data[rxi];
 	      for (spi = 0; spi < spexp->cnt; spi++) {
-	        spex = spexp->data[spi];
-	        sp_esf = pow(0.75,spex);
+		spex = spexp->data[spi];
+		sp_esf = ESF_SEL(sp_esf, pow(0.75, spex));
 		for (nsi = 0; nsi < nsexp->cnt; nsi++) {
 		    uint fp, fn;
 		    nsex = nsexp->data[nsi];
-		    ns_esf = pow(0.75, nsex);
+		    ns_esf = ESF_SEL(ns_esf, pow(0.75, nsex));
 
 		    /* save parms */
 		    r = &results[cnt++];
@@ -1546,7 +1578,7 @@ static rc_t bogotune(void)
 
 		    spam_cutoff = 0.01;
 		    score_ns(ns_scores);	/* scores in descending order */
-		    
+
 		    /* Determine spam_cutoff and false_pos */
 		    for (fp = target; fp < ns_cnt; fp += 1) {
 			spam_cutoff = ns_scores[fp-1];
@@ -1604,14 +1636,15 @@ static rc_t bogotune(void)
 	robs = rsval->data[best->rsi];
 	robx = rxval->data[best->rxi];
 	min_dev = mdval->data[best->mdi];
-	spex = spexp->data[best->spi]; sp_esf = pow(0.75, spex);
-	nsex = nsexp->data[best->nsi]; ns_esf = pow(0.75, nsex);
+
+	spex = spexp->data[best->spi]; sp_esf = ESF_SEL(sp_esf, pow(0.75, spex));
+	nsex = nsexp->data[best->nsi]; ns_esf = ESF_SEL(ns_esf, pow(0.75, nsex));
 
 	printf(
     "Minimum found at s %6.4f, md %5.3f, x %5.3f, spesf %8.6f, nsesf %8.6f\n",
-	        robs, min_dev, robx, sp_esf, ns_esf);
+    		robs, min_dev, robx, sp_esf, ns_esf);
 	printf("        fp %u (%6.4f%%), fn %u (%6.4f%%)\n",
-		best->fp, best->fp*100.0/ns_cnt, 
+		best->fp, best->fp*100.0/ns_cnt,
 		best->fn, best->fn*100.0/sp_cnt);
 	printf("\n");
 
@@ -1675,6 +1708,8 @@ int main(int argc, char **argv) /*@globals errno,stderr,stdout@*/
 
     bogotune_free();
 
+    ds_cleanup();
+
     exit(exitcode);
 }
 
@@ -1683,7 +1718,7 @@ static void bt_exit(void)
     return;
 }
 
-static void show_elapsed_time(int beg, int end, uint cnt, double val, 
+static void show_elapsed_time(int beg, int end, uint cnt, double val,
 			      const char *lbl1, const char *lbl2)
 {
     int tm = end - beg;
