@@ -1,4 +1,4 @@
-/* $Id: datastore_db.c,v 1.107 2004/06/19 19:17:56 relson Exp $ */
+/* $Id: datastore_db.c,v 1.111 2004/09/29 03:28:04 relson Exp $ */
 
 /*****************************************************************************
 
@@ -49,7 +49,6 @@ static DB_ENV *dbe; /* libdb environment, if in use, NULL otherwise */
 static const DBTYPE dbtype = DB_BTREE;
 
 static bool init = false;
-bool create_flag = false;		/* For datastore.c (to add .WORDLIST_VERSION) */
 
 typedef struct {
     char	*path;
@@ -70,9 +69,6 @@ typedef struct {
 /* dummy infrastructure, to be expanded by environment
  * or transactional initialization/shutdown */
 
-static int db_init(void);
-static void db_cleanup(void);
-
 /* Function definitions */
 
 /** translate BerkeleyDB \a flags bitfield back to symbols */
@@ -201,6 +197,7 @@ static void check_db_version(void)
  * limit. errors from the system are ignored, no warning then.
  */
 static void check_fsize_limit(int fd, uint32_t pagesize) {
+#ifndef __EMX__
     struct stat st;
     struct rlimit rl;
 
@@ -225,6 +222,7 @@ static void check_fsize_limit(int fd, uint32_t pagesize) {
 	    print_error(__FILE__, __LINE__, "         data base corruption.");
 	}
     }
+#endif
 }
 
 /* The old, pre-3.3 API will not fill in the page size with
@@ -285,10 +283,6 @@ void *db_open(const char *path, const char *name, dbmode_t open_mode)
     size_t idx;
     uint32_t retryflags[] = { 0, DB_NOMMAP };
 
-    db_init();
-
-    if (!init) abort();
-
     check_db_version();
 
     if (open_mode & DS_READ )
@@ -574,8 +568,6 @@ void db_close(void *vhandle, bool nosync)
 	print_error(__FILE__, __LINE__, "(db) db_close err: %d, %s", ret, db_strerror(ret));
 
     dbh_free(handle);
-
-    db_cleanup();
 }
 
 
@@ -671,12 +663,10 @@ const char *db_str_err(int e) {
 /* dummy infrastructure, to be expanded by environment
  * or transactional initialization/shutdown */
 
-static int db_init(void) {
+int db_init(void) {
     char *t;
     int cdb_alldb = 1;
 
-    assert(bogohome);
-
     if (bogohome && getenv("BOGOFILTER_CONCURRENT_DATA_STORE")) {
 	int ret = db_env_create(&dbe, 0);
 	if (ret != 0) {
@@ -705,7 +695,7 @@ static int db_init(void) {
     return 0;
 }
 
-static void db_cleanup(void) {
+void db_cleanup(void) {
     if (!init)
 	return;
     if (dbe)
