@@ -1,4 +1,4 @@
-/* $Id: format.c,v 1.37 2004/06/14 23:45:44 relson Exp $ */
+/* $Id: format.c,v 1.42 2004/07/15 00:56:19 m-a Exp $ */
 
 /*****************************************************************************
 
@@ -52,6 +52,8 @@ const char *unsure_subject_tag = NULL;			/* used in passthrough mode */
 /*
 **	formatting characters:
 **
+**	    XXX FIXME XXX this list is outdated and needs revision or removal
+**
 **	    h - spam_header_name, e.g. "X-Bogosity"
 **
 **	    c - classification, e.g. Yes/No, Spam/Ham/Unsure, +/-/?
@@ -199,13 +201,11 @@ static size_t format_float(char *dest, double src,
 
 static size_t format_string(char *dest, const char *src, int min, int prec, int flags, const char *destend)
 {
-    int s_len = (int) strlen(src);
-    int len;
-    if (s_len > INT_MAX) {
-	fprintf(stderr, "cannot handle string length (%lu) above %d, aborting\n", (unsigned long)s_len, INT_MAX);
+    int len = (int) strlen(src);
+    if (len > INT_MAX) {
+	fprintf(stderr, "cannot handle string length (%lu) above %d, aborting\n", (unsigned long)len, INT_MAX);
 	internal_error;
     }
-    len = s_len;
     (void)min; /* kill compiler warning */
 
     if (flags & F_PREC && prec < len)
@@ -318,6 +318,9 @@ char *convert_format_to_string(char *buff, size_t size, const char *format)
 	    case '%':
 		*buff++ = '%';
 		break;
+	    case 'A':		/* A - Message Address */
+		buff += format_string(buff, msg_addr ? (const char *)msg_addr->text : "UNKNOWN", 0, prec, flags, end);
+		break;
 	    case 'c':		/* c - classification, e.g. Yes/No, Spam/Ham/Unsure, or YN, SHU, +-? */
 	    {
 		const char *val = spamicity_tags[status];
@@ -337,7 +340,12 @@ char *convert_format_to_string(char *buff, size_t size, const char *format)
 		buff += format_date(buff, end);
 		break;
 	    }
-
+	    case 'I':		/* M - Message ID */
+		buff += format_string(buff, msg_id ? (const char *)msg_id->text : "UNKNOWN", 0, prec, flags, end);
+		break;
+	    case 'Q':		/* Q - Queue ID */
+		buff += format_string(buff, queue_id ? (const char *)queue_id->text : "UNKNOWN", 0, prec, flags, end);
+		break;
 	    case 'p':		/* p - spamicity as a probability */
 	    {
 		const char *f = spamicity_formats[status];
@@ -356,9 +364,6 @@ char *convert_format_to_string(char *buff, size_t size, const char *format)
 	    case 'h':		/* h - spam_header_name, e.g. "X-Bogosity" */
 		buff += format_string(buff, spam_header_name, 0, prec, flags, end);
 		break;
-	    case 'I':		/* I - received IP address */
-		buff += format_string(buff, ipaddr->text, 0, prec, flags, end);
-		break;
 	    case 'l':		/* l - logging tag */
 		buff += format_string(buff, logtag, 0, prec, flags, end);
 		break;
