@@ -1,4 +1,4 @@
-/* $Id: globals.c,v 1.60 2004/05/09 14:42:04 relson Exp $ */
+/* $Id: globals.c,v 1.62 2004/08/22 15:25:04 relson Exp $ */
 
 /*****************************************************************************
 
@@ -28,10 +28,10 @@ bool	mbox_mode;			/* '-M' */
 bool	replace_nonascii_characters;	/* '-n' */
 bool	passthrough;			/* '-p' */
 bool	quiet;				/* '-q' */
-bool	query;				/* '-Q' */
-int	Rtable = 0;			/* '-R' */
+int	query;				/* '-Q' */
+bool	Rtable;				/* '-R' */
 bool	terse;				/* '-t' */
-int	bogotest = 0;			/* '-X', env("BOGOTEST") */
+int	bogotest;			/* '-X', env("BOGOTEST") */
 int	verbose;			/* '-v' */
 
 /* config file options */
@@ -62,7 +62,7 @@ FILE	*fpo;
 uint	db_cachesize = DB_CACHESIZE;	/* in MB */
 enum	passmode passmode;		/* internal */
 bool	msg_count_file = false;
-const char *progtype = NULL;
+char	*progtype = NULL;
 bool	unsure_stats = false;		/* true if print stats for unsures */
 
 run_t run_type = RUN_UNKNOWN;
