@@ -1,4 +1,4 @@
-/* $Id: lexer.c,v 1.96 2004/06/26 04:14:35 relson Exp $ */
+/* $Id: lexer.c,v 1.103 2004/10/13 23:18:38 relson Exp $ */
 
 /*
  * NAME
@@ -18,6 +18,7 @@
 #include "charset.h"
 #include "error.h"
 #include "lexer.h"
+#include "memstr.h"
 #include "mime.h"
 #include "msgcounts.h"
 #include "qp.h"
@@ -28,7 +29,8 @@
 
 /* Global Variables */
 
-int yylineno;
+extern int yylineno;
+
 bool msg_header = true;
 bool have_body  = false;
 lexer_t *lexer = NULL;
@@ -54,6 +56,14 @@ static int skip_folded_line(buff_t *buff);
 
 /* Function Definitions */
 
+void lexer_init(void)
+{
+    mime_reset();
+    token_init();
+    lexer_v3_init(NULL);
+    init_charset_table(charset_default, true);
+}
+
 static void lexer_display_buffer(buff_t *buff)
 {
     fprintf(dbgout, "*** %2d %c%c %2ld ",
@@ -116,6 +126,8 @@ static int yy_get_new_line(buff_t *buff)
     /* skip spam_header ("X-Bogosity:") lines */
     while (msg_header
 	   && count != EOF
+	   /* don't skip if inside message/rfc822 */
+	   && msg_state == msg_state->parent
 	   && memcmp(buff->t.text,spam_header_name,hdrlen) == 0) {
 	count = skip_folded_line(buff);
     }
@@ -192,7 +204,7 @@ static int skip_folded_line(buff_t *buff)
 	if (!isspace(buff->t.text[0])) 
 	    return count;
 	/* Check for empty line which terminates message header */
-	if (is_eol(buff->t.text, count))
+	if (is_eol((char *)buff->t.text, count))
 	    return count;
     }
 
@@ -281,22 +293,28 @@ int yyinput(byte *buf, size_t used, size_t size)
 
 size_t text_decode(word_t *w)
 {
-    byte *beg = w->text;
-    byte *fin = beg + w->leng;
-    byte *txt = (byte *) strstr(beg, "=?");	/* skip past leading text */
-    uint size = (uint) (txt - beg);
+    byte *const beg = w->text;		/* base pointer, fixed */
+    byte *const fin = beg + w->leng;	/* end+1 position */
+
+    byte *txt = (byte *) memstr(w->text, w->leng, "=?");	/* input position */
+    uint size = (uint) (txt - beg);				/* output offset */
+
+    if (txt == NULL)
+	return w->leng;
 
     while (txt < fin) {
 	word_t n;
-	byte *typ = (byte *) strchr(txt+2, '?');/* Encoding type - 'B' or 'Q' */
-	byte *tmp = typ + 3;
-	byte *end = (byte *) strstr(tmp, "?=");	/* last byte of encoded word  */
-	uint len = end - tmp;
-	bool copy;
+	byte *typ, *tmp, *end;
+	uint len;
+	bool adjacent;
+
+	typ = (byte *) memchr((char *)txt+2, '?', fin-(txt+2));	/* Encoding type - 'B' or 'Q' */
+	tmp = typ + 3;						/* start of encoded word */
+	end = (byte *) memstr((char *)tmp, fin-tmp, "?=");	/* last byte of encoded word  */
+	len = end - tmp;
 
 	n.text = tmp;				/* Start of encoded word */
 	n.leng = len;				/* Length of encoded word */
-	n.text[n.leng] = (byte) '\0';
 
 	if (DEBUG_LEXER(2)) {
 	    fputs("***  ", dbgout);
@@ -310,13 +328,12 @@ size_t text_decode(word_t *w)
 		len = base64_decode(&n);	/* decode base64 */
 	    break;
 	case 'q':
-	    if (qp_validate(&n))
-		len = qp_decode(&n);		/* decode quoted-printable */
+	    if (qp_validate(&n, RFC2047))
+		len = qp_decode(&n, RFC2047);	/* decode quoted-printable */
 	    break;
 	}
 
 	n.leng = len;
-	n.text[n.leng] = '\0';
 
 	if (DEBUG_LEXER(3)) {
 	    fputs("***  ", dbgout);
@@ -324,28 +341,38 @@ size_t text_decode(word_t *w)
 	    fputs("\n", dbgout);
 	}
 
+	/* move decoded word to where the encoded used to be */
 	memmove(beg+size, n.text, len+1);
-	size += len;
-	txt = end + 2;
+	size += len;	/* bump output pointer */
+	txt = end + 2;	/* skip ?= trailer */
 	if (txt >= fin)
 	    break;
-	end = (byte *) strstr(txt, "=?");
-	copy = end != NULL;
 
-	if (copy) {
+	/* check for next encoded word */
+	end = (byte *) memstr((char *)txt, fin-txt, "=?");
+	adjacent = end != NULL;
+
+	/* clear adjacent flag if non-whitespace character found between
+	 * adjacent encoded words */
+	if (adjacent) {
 	    tmp = txt;
-	    while (copy && tmp < end) {
+	    while (adjacent && tmp < end) {
 		if (isspace(*tmp))
 		    tmp += 1;
 		else
-		    copy = false;
+		    adjacent = false;
 	    }
 	}
 
-	if (copy)
+	/* we have a next encoded word and we've had only whitespace
+	 * between the current and the next */
+	if (adjacent)
+	    /* just skip whitespace */
 	    txt = end;
-	else while (txt < end)
-	    beg[size++] = *txt++;
+	else
+	    /* copy everything that was between the encoded words */
+	    while (txt < end)
+		beg[size++] = *txt++;
     }
 
     return size;
