@@ -1,4 +1,4 @@
-/* $Id: paths.h,v 1.10 2004/05/18 00:22:52 relson Exp $ */
+/* $Id: paths.h,v 1.11 2004/08/07 12:31:29 relson Exp $ */
 
 /*****************************************************************************
 
@@ -14,7 +14,7 @@ NAME:
 
 /* Path Definitions */
 
-#if !defined(__OS2__) && !defined(__riscos__)
+#if !defined(_OS2_) && !defined(__riscos__)
   #define BOGODIR ".bogofilter"
 #else
   #define BOGODIR "bogofilter"
