@@ -1,4 +1,4 @@
-/* $Id: score.c,v 1.22 2004/06/09 16:38:46 relson Exp $ */
+/* $Id: score.c,v 1.25 2004/10/19 23:49:47 relson Exp $ */
 
 /*****************************************************************************
 
@@ -139,7 +139,7 @@ static void lookup(const word_t *token, wordcnts_t *cnts)
     return;
 }
 
-double msg_lookup_and_score(const word_t *token, wordcnts_t *cnts)
+static double msg_lookup_and_score(const word_t *token, wordcnts_t *cnts)
 {
     double prob;
 
@@ -369,13 +369,13 @@ double get_spamicity(size_t robn, FLOAT P, FLOAT Q)
 	score.robn = robn;
 
 	/* convert to natural logs */
-        score.p_ln = (log(P.mant) + P.exp * ln2) * sp_esf;  /* invlogsum */
-        score.q_ln = (log(Q.mant) + Q.exp * ln2) * ns_esf;  /* logsum */
+	score.p_ln = (log(P.mant) + P.exp * ln2) * sp_esf;	/* invlogsum */
+	score.q_ln = (log(Q.mant) + Q.exp * ln2) * ns_esf;	/* logsum */
 
-        score.p_pr = prbf(-2.0 * score.p_ln, sp_df);        /* compute P */
-        score.q_pr = prbf(-2.0 * score.q_ln, ns_df);        /* compute Q */
+	score.p_pr = prbf(-2.0 * score.p_ln, sp_df);		/* compute P */
+	score.q_pr = prbf(-2.0 * score.q_ln, ns_df);		/* compute Q */
   
-        if (!fBogotune && sp_esf == 1.0 && ns_esf == 1.0) {
+        if (!fBogotune && sp_esf >= 1.0 && ns_esf >= 1.0) {
             score.spamicity = (1.0 + score.q_pr - score.p_pr) / 2.0;
         } else if (score.q_pr < DBL_EPSILON && score.p_pr < DBL_EPSILON) {
             score.spamicity = 0.5;
