@@ -1,4 +1,4 @@
-/* $Id: token.h,v 1.15 2004/06/14 23:45:45 relson Exp $ */
+/* $Id: token.h,v 1.18 2004/07/15 00:56:19 m-a Exp $ */
 
 /*****************************************************************************
 
@@ -13,7 +13,9 @@ NAME:
 #include "lexer.h"
 
 extern word_t *yylval;
-extern word_t *ipaddr;		/* First IP Address in Received: statement */
+extern word_t *msg_addr;	/* First IP Address in Received: statement */
+extern word_t *msg_id;		/* Message ID */
+extern word_t *queue_id;	/* Message's first Queue ID */
 
 extern token_t get_token(void);
 
