@@ -1,4 +1,4 @@
-/* $Id: wordlists.c,v 1.72 2004/06/11 01:27:37 relson Exp $ */
+/* $Id: wordlists.c,v 1.73 2004/08/07 12:31:29 relson Exp $ */
 
 #include "common.h"
 
@@ -40,6 +40,9 @@ static bool open_wordlist(wordlist_t *list, dbmode_t mode)
 	switch(err) {
 	    /* F_SETLK can't obtain lock */
 	case EAGAIN:
+#ifdef __EMX__
+	case EACCES:
+#endif
 	    /* case EACCES: */
 	    rand_sleep(MIN_SLEEP, MAX_SLEEP);
 	    retry = true;
